import { Component, OnInit, SecurityContext, ViewChild } from '@angular/core';
import { DomSanitizer } from '@angular/platform-browser';
import { ActivatedRoute, Router } from '@angular/router';
import { User } from '../../../models/user.model';
import { NotificationService } from '../../../services/notification.service';
import { OrientationService } from '../../../services/orientation.service';
import { TextareaComponent } from '../../../shared/components/textarea/textarea.component';
import { ProfileService } from '../../services/profile.service';

@Component({
  selector: 'app-orientation-details',
  templateUrl: './orientation-details.component.html',
  styleUrls: ['./orientation-details.component.scss'],
})
export class OrientationDetailsComponent implements OnInit {
  constructor(
    private route: ActivatedRoute,
    public orientationService: OrientationService,
    private notificationService: NotificationService,
    private profileService: ProfileService,
    private sanitizer: DomSanitizer,
    private router: Router,
  ) {
    const navigation = this.router.getCurrentNavigation();
    if (navigation?.extras?.state) {
      this.from = navigation.extras.state['from'];
    }
  }

  @ViewChild('comment', { static: false }) commentTextarea: TextareaComponent;

  public orientation;
  public isModalOpenned = false;
  public userProfile: User;
  public from: string;
  public processingComment = '';
  public selectedStatus = '';
  public isClosed: boolean;
  private closedStatuses = ['completed', 'uncompleted', 'expired'];

  ngOnInit(): void {
    this.route.params.subscribe((urlParams) => {
      const orientationId = urlParams.id || '';
      if (orientationId) {
        this.orientationService.getOrientation(orientationId).subscribe((orientation) => {
          this.orientation = orientation;
          this.acknowledgeNewOrientation();
          this.isOrientationClosed();
          if (this.isClosed) {
            this.selectedStatus = this.orientation.status;
          }
        });
      }
    });

    // if coming from the email (with a URL like /details-orientation/[orientationID]#to)
    const hash = this.router.parseUrl(this.router.url).fragment;
    if (hash === 'to') {
      this.from = 'todoOrientations';
    }

    this.profileService.getProfile().then((profile) => {
      this.userProfile = new User(profile);
    });
  }

  public goBack(): void {
    history.back();
  }

  public showModal(): void {
    this.isModalOpenned = true;
  }

  public closeOrientation(event, isClosed): void {
    if (event) {
      const comment = this.commentTextarea?.value || '';
      const sanitizedHtml = this.sanitizer.sanitize(SecurityContext.HTML, comment);
      const sanitizedComment = this.extractPlainText(sanitizedHtml || '');
      const patchData: any = {
        status: this.selectedStatus,
        action: null,
        processingComment: sanitizedComment,
      };
      // Only add closingDate if orientation is not already closed
      if (!isClosed) {
        patchData.closingDate = new Date();
      }
      this.patchOrientation(patchData);
    }
    this.isModalOpenned = false;
  }

  private extractPlainText(html: string): string {
    const tempElement = document.createElement('div');
    tempElement.innerHTML = html;
    return tempElement.textContent || '';
  }

  public acknowledgeNewOrientation(): void {
    if (this.orientation.status === 'new' && this.from === 'todoOrientations') {
      this.patchOrientation({ status: 'waiting', action: 'defineAppointment' });
    }
  }

  public setAppointmentDate(event: Event): void {
    const target = event.target as HTMLInputElement;
    const date = new Date(target.value);
    if (!isNaN(date.getTime())) {
      this.patchOrientation({ appointmentDate: date });
    }
  }

  get isValidateDisabled(): boolean {
    return !this.selectedStatus || !this.commentTextarea?.value?.trim();
  }

  public isOrientationClosed(): void {
    this.isClosed = this.closedStatuses.includes(this.orientation.status);
  }

  private patchOrientation(updatedFields): void {
    this.orientationService.patchOrientation(this.orientation._id, updatedFields).subscribe({
      next: (orientation) => {
        this.notificationService.showSuccess("L'orientation a bien été mise à jour.");
        // Display the updated values on the orientation details page
        this.orientation = orientation;
      },
      error: () => {
        this.notificationService.showErrorPleaseRetry(
          "Une erreur s'est produite lors de la mise à jour de l'orientation.",
        );
      },
    });
  }
}
