import { Component, EventEmitter, HostListener, Input, OnInit, Output } from '@angular/core';
import { CategoryEnum } from '../../../shared/enum/category.enum';
import { Category } from '../../models/category.model';
import { Module } from '../../models/module.model';
import { SearchService } from '../../services/search.service';

@Component({
  selector: 'app-more-filters',
  templateUrl: './more-filters.component.html',
  styleUrls: ['./more-filters.component.scss'],
})
export class MoreFiltersComponent implements OnInit {
  constructor(public searchService: SearchService) {}

  /** Display content as modal */
  @Input() public isModal = false;
  /** Categories list to display */
  @Input() public categories: Category[];
  /** Checked modules list */
  @Input() public checkedModules: Module[] = [];
  /** Display Radio buttons instead of checkboxes */
  @Input() isRadio = false;
  @Output() selectEvent = new EventEmitter();
  @Output() closeEvent = new EventEmitter();
  public categoryEnum = CategoryEnum;
  public isMobile = false;

  /** Checked but not yet validated modules */
  public pendingCheckedModules: Module[] = [];
  ngOnInit(): void {
    // Manage checkbox
    this.pendingCheckedModules = this.checkedModules.slice();
    this.checkMobile();
  }

  // Management of the checkbox event (Check / Uncheck)
  public onCheckboxChange(event: Event, categoryId: string, moduleId: string, text: string): void {
    // Handle mouse or keyboard event
    const checked: boolean = (event.target as HTMLInputElement).checked;
    if (checked) {
      this.pendingCheckedModules.push(new Module(moduleId, categoryId, text));
    } else {
      // Check if the module is present in the list and remove it
      const index = this.searchService.getIndex(this.pendingCheckedModules, moduleId, categoryId);
      if (index > -1) {
        this.pendingCheckedModules.splice(index, 1);
      }
    }
  }

  // Management of the radio event
  public onRadioChange(categoryId: string, moduleId: string, text: string): void {
    this.pendingCheckedModules = [new Module(moduleId, categoryId, text)];
  }

  // Clear only filters in the current modal
  public clearFilters(): void {
    this.categories.forEach((categ) => {
      categ.modules.forEach((module) => {
        const index = this.searchService.getIndex(this.pendingCheckedModules, module.id, categ.id);
        if (index > -1) {
          this.pendingCheckedModules.splice(index, 1);
        }
      });
    });
    this.emitModules(this.pendingCheckedModules);
  }

  // Sends an array containing all modules
  public emitModules(m: Module[]): void {
    this.selectEvent.emit(m);
  }

  public closeModal(): void {
    this.closeEvent.emit();
  }

  public handleCategoryCheckBox(event, category: Category): void {
    const modules = this.categories.filter((c: Category) => c.id === category.id)[0].modules;
    if (event.target.checked) {
      for (const module of modules) {
        if (this.searchService.getIndex(this.pendingCheckedModules, module.id, category.id) === -1) {
          this.pendingCheckedModules.push(new Module(module.id, category.id, module.displayText));
        }
      }
    } else {
      for (const module of modules) {
        if (this.searchService.getIndex(this.pendingCheckedModules, module.id, category.id) > -1) {
          this.pendingCheckedModules.splice(
            this.searchService.getIndex(this.pendingCheckedModules, module.id, category.id),
            1,
          );
        }
      }
    }
  }

  public getCategoryCheckboxStatus(c: Category): string {
    const selectedModule: Module[] = this.pendingCheckedModules.filter((m) => m.name === c.id);
    if (selectedModule.length === 0) {
      return 'unchecked';
    } else if (selectedModule.length === c.modules.length) {
      return 'checked';
    } else if (selectedModule.length && selectedModule.length > 0) {
      return 'halfChecked';
    }
  }

  // Checks if size of screen is less than 576px to display the modal with wide buttons instead of dropdown menu
  @HostListener('window:resize', ['$event'])
  onResize(event) {
    this.checkMobile();
  }
  checkMobile() {
    this.isMobile = window.innerWidth <= 576;
  }
}
